@@ -588,7 +588,9 @@ impl ImapConnection {
     pub fn connect<'a>(&'a mut self) -> Pin<Box<dyn Future<Output = Result<()>> + Send + 'a>> {
         Box::pin(async move {
             if let (time, ref mut status @ Ok(())) = *self.uid_store.is_online.lock().unwrap() {
-                if SystemTime::now().duration_since(time).unwrap_or_default() >= IMAP_PROTOCOL_TIMEOUT {
+                if SystemTime::now().duration_since(time).unwrap_or_default()
+                    >= IMAP_PROTOCOL_TIMEOUT
+                {
                     let err = MeliError::new("Connection timed out").set_kind(ErrorKind::Timeout);
                     *status = Err(err.clone());
                     self.stream = Err(err);
@@ -978,45 +980,38 @@ impl ImapConnection {
 
     pub async fn unselect(&mut self) -> Result<()> {
         match self.stream.as_mut()?.current_mailbox.take() {
-            MailboxSelection::Examine(_) |
-                MailboxSelection::Select(_) => {
-                    let mut response = Vec::with_capacity(8 * 1024);
-                    if self
-                        .uid_store
-                            .capabilities
-                            .lock()
-                            .unwrap()
-                            .iter()
-                            .any(|cap| cap.eq_ignore_ascii_case(b"UNSELECT"))
+            MailboxSelection::Examine(_) | MailboxSelection::Select(_) => {
+                let mut response = Vec::with_capacity(8 * 1024);
+                if self
+                    .uid_store
+                    .capabilities
+                    .lock()
+                    .unwrap()
+                    .iter()
+                    .any(|cap| cap.eq_ignore_ascii_case(b"UNSELECT"))
+                {
+                    self.send_command(b"UNSELECT").await?;
+                    self.read_response(&mut response, RequiredResponses::empty())
+                        .await?;
+                } else {
+                    /* `RFC3691 - UNSELECT Command` states: "[..] IMAP4 provides this
+                     * functionality (via a SELECT command with a nonexistent mailbox name or
+                     * reselecting the same mailbox with EXAMINE command)[..]
+                     */
+                    let mut nonexistent = "blurdybloop".to_string();
                     {
-                        self.send_command(b"UNSELECT").await?;
-                        self.read_response(&mut response, RequiredResponses::empty())
-                            .await?;
-                        } else {
-                            /* `RFC3691 - UNSELECT Command` states: "[..] IMAP4 provides this
-                             * functionality (via a SELECT command with a nonexistent mailbox name or
-                             * reselecting the same mailbox with EXAMINE command)[..]
-                             */
-                            let mut nonexistent = "blurdybloop".to_string();
-                            {
-                                let mailboxes = self.uid_store.mailboxes.lock().await;
-                                while mailboxes.values().any(|m| m.imap_path() == nonexistent) {
-                                    nonexistent.push('p');
-                                }
-                            }
-                            self.send_command(
-                                format!(
-                                    "SELECT \"{}\"",
-                                    nonexistent
-                                )
-                                .as_bytes(),
-                            )
-                                .await?;
-                            self.read_response(&mut response, RequiredResponses::NO_REQUIRED)
-                                .await?;
+                        let mailboxes = self.uid_store.mailboxes.lock().await;
+                        while mailboxes.values().any(|m| m.imap_path() == nonexistent) {
+                            nonexistent.push('p');
                         }
+                    }
+                    self.send_command(format!("SELECT \"{}\"", nonexistent).as_bytes())
+                        .await?;
+                    self.read_response(&mut response, RequiredResponses::NO_REQUIRED)
+                        .await?;
                 }
-            MailboxSelection::None => {},
+            }
+            MailboxSelection::None => {}
         }
         Ok(())
     }
