@@ -47,10 +47,46 @@ use std::sync::{Arc, Mutex};
 use std::time::{Duration, Instant};
 pub type UID = usize;
 
+macro_rules! get_conf_val {
+    ($s:ident[$var:literal]) => {
+        $s.extra.get($var).ok_or_else(|| {
+            MeliError::new(format!(
+                "Configuration error ({}): NNTP connection requires the field `{}` set",
+                $s.name.as_str(),
+                $var
+            ))
+        })
+    };
+    ($s:ident[$var:literal], $default:expr) => {
+        $s.extra
+            .get($var)
+            .map(|v| {
+                <_>::from_str(v).map_err(|e| {
+                    MeliError::new(format!(
+                        "Configuration error ({}) NNTP: Invalid value for field `{}`: {}\n{}",
+                        $s.name.as_str(),
+                        $var,
+                        v,
+                        e
+                    ))
+                })
+            })
+            .unwrap_or_else(|| Ok($default))
+    };
+}
+
 pub static SUPPORTED_CAPABILITIES: &[&str] = &[
     #[cfg(feature = "deflate_compression")]
     "COMPRESS DEFLATE",
     "VERSION 2",
+    "NEWNEWS",
+    "POST",
+    "OVER",
+    "OVER MSGID",
+    "READER",
+    "STARTTLS",
+    "HDR",
+    "AUTHINFO USER",
 ];
 
 #[derive(Debug, Clone)]
@@ -74,6 +110,7 @@ pub struct UIDStore {
     account_name: Arc<String>,
     offline_cache: bool,
     capabilities: Arc<Mutex<Capabilities>>,
+    message_id_index: Arc<Mutex<HashMap<String, EnvelopeHash>>>,
     hash_index: Arc<Mutex<HashMap<EnvelopeHash, (UID, MailboxHash)>>>,
     uid_index: Arc<Mutex<HashMap<(MailboxHash, UID), EnvelopeHash>>>,
 
@@ -95,6 +132,7 @@ impl UIDStore {
             event_consumer,
             offline_cache: false,
             capabilities: Default::default(),
+            message_id_index: Default::default(),
             hash_index: Default::default(),
             uid_index: Default::default(),
             mailboxes: Arc::new(FutureMutex::new(Default::default())),
@@ -131,6 +169,7 @@ impl MailBackend for NntpType {
                 )
             })
             .collect::<Vec<(String, MailBackendExtensionStatus)>>();
+        let mut supports_submission = false;
         let NntpExtensionUse {
             #[cfg(feature = "deflate_compression")]
             deflate,
@@ -138,6 +177,10 @@ impl MailBackend for NntpType {
         {
             for (name, status) in extensions.iter_mut() {
                 match name.as_str() {
+                    s if s.eq_ignore_ascii_case("POST") => {
+                        supports_submission = true;
+                        *status = MailBackendExtensionStatus::Enabled { comment: None };
+                    }
                     "COMPRESS DEFLATE" => {
                         #[cfg(feature = "deflate_compression")]
                         {
@@ -171,7 +214,7 @@ impl MailBackend for NntpType {
             supports_search: false,
             extensions: Some(extensions),
             supports_tags: false,
-            supports_submission: false,
+            supports_submission,
         }
     }
 
@@ -201,8 +244,88 @@ impl MailBackend for NntpType {
         }))
     }
 
-    fn refresh(&mut self, _mailbox_hash: MailboxHash) -> ResultFuture<()> {
-        Err(MeliError::new("Unimplemented."))
+    fn refresh(&mut self, mailbox_hash: MailboxHash) -> ResultFuture<()> {
+        let uid_store = self.uid_store.clone();
+        let connection = self.connection.clone();
+        Ok(Box::pin(async move {
+            /* To get updates, either issue NEWNEWS if it's supported by the server, and fallback
+             * to OVER otherwise */
+            let mbox: NntpMailbox = uid_store.mailboxes.lock().await.get(&mailbox_hash).map(std::clone::Clone::clone).ok_or_else(|| MeliError::new(format!("Mailbox with hash {} not found in NNTP connection, this could possibly be a bug or it was deleted.", mailbox_hash)))?;
+            let latest_article: Option<crate::UnixTimestamp> =
+                mbox.latest_article.lock().unwrap().clone();
+            let (over_msgid_support, newnews_support): (bool, bool) = {
+                let caps = uid_store.capabilities.lock().unwrap();
+
+                (
+                    caps.iter().any(|c| c.eq_ignore_ascii_case("OVER MSGID")),
+                    caps.iter().any(|c| c.eq_ignore_ascii_case("NEWNEWS")),
+                )
+            };
+            let mut res = String::with_capacity(8 * 1024);
+            let mut conn = timeout(Some(Duration::from_secs(60 * 16)), connection.lock()).await?;
+            if let Some(mut latest_article) = latest_article {
+                let timestamp = latest_article - 10 * 60;
+                let datetime_str =
+                    crate::datetime::timestamp_to_string(timestamp, Some("%Y%m%d %H%M%S"), true);
+
+                if newnews_support {
+                    conn.send_command(
+                        format!("NEWNEWS {} {}", &mbox.nntp_path, datetime_str).as_bytes(),
+                    )
+                    .await?;
+                    conn.read_response(&mut res, true, &["230 "]).await?;
+                    let message_ids = {
+                        let message_id_lck = uid_store.message_id_index.lock().unwrap();
+                        res.split_rn()
+                            .skip(1)
+                            .map(|s| s.trim())
+                            .filter(|msg_id| !message_id_lck.contains_key(*msg_id))
+                            .map(str::to_string)
+                            .collect::<Vec<String>>()
+                    };
+                    if message_ids.is_empty() || !over_msgid_support {
+                        return Ok(());
+                    }
+                    let mut env_hash_set: BTreeSet<EnvelopeHash> = Default::default();
+                    for msg_id in message_ids {
+                        conn.send_command(format!("OVER {}", msg_id).as_bytes())
+                            .await?;
+                        conn.read_response(&mut res, true, &["224 "]).await?;
+                        let mut message_id_lck = uid_store.message_id_index.lock().unwrap();
+                        let mut hash_index_lck = uid_store.hash_index.lock().unwrap();
+                        let mut uid_index_lck = uid_store.uid_index.lock().unwrap();
+                        for l in res.split_rn().skip(1) {
+                            let (_, (num, env)) = protocol_parser::over_article(&l)?;
+                            env_hash_set.insert(env.hash());
+                            message_id_lck.insert(env.message_id_display().to_string(), env.hash());
+                            hash_index_lck.insert(env.hash(), (num, mailbox_hash));
+                            uid_index_lck.insert((mailbox_hash, num), env.hash());
+                            latest_article = std::cmp::max(latest_article, env.timestamp);
+                            (uid_store.event_consumer)(
+                                uid_store.account_hash,
+                                crate::backends::BackendEvent::Refresh(RefreshEvent {
+                                    mailbox_hash,
+                                    account_hash: uid_store.account_hash,
+                                    kind: RefreshEventKind::Create(Box::new(env)),
+                                }),
+                            );
+                        }
+                    }
+                    {
+                        let f = &uid_store.mailboxes.lock().await[&mailbox_hash];
+                        *f.latest_article.lock().unwrap() = Some(latest_article);
+                        f.exists
+                            .lock()
+                            .unwrap()
+                            .insert_existing_set(env_hash_set.clone());
+                        f.unseen.lock().unwrap().insert_existing_set(env_hash_set);
+                    }
+                    return Ok(());
+                }
+            }
+            //conn.select_group(mailbox_hash, false, &mut res).await?;
+            Ok(())
+        }))
     }
 
     fn mailboxes(&self) -> ResultFuture<HashMap<MailboxHash, Mailbox>> {
@@ -240,7 +363,7 @@ impl MailBackend for NntpType {
     }
 
     fn watch(&self) -> ResultFuture<()> {
-        Err(MeliError::new("Unimplemented."))
+        Err(MeliError::new("Unimplemented.").set_kind(ErrorKind::NotImplemented))
     }
 
     fn operation(&self, env_hash: EnvelopeHash) -> Result<Box<dyn BackendOp>> {
@@ -354,6 +477,39 @@ impl MailBackend for NntpType {
     ) -> ResultFuture<SmallVec<[EnvelopeHash; 512]>> {
         Err(MeliError::new("Unimplemented."))
     }
+
+    fn submit(
+        &self,
+        bytes: Vec<u8>,
+        mailbox_hash: Option<MailboxHash>,
+        _flags: Option<Flag>,
+    ) -> ResultFuture<()> {
+        let connection = self.connection.clone();
+        Ok(Box::pin(async move {
+            match timeout(Some(Duration::from_secs(60 * 16)), connection.lock()).await {
+                Ok(mut conn) => {
+                    match &conn.stream {
+                        Ok(stream) => {
+                            if !stream.supports_submission {
+                                return Err(MeliError::new("Server prohibits posting."));
+                            }
+                        }
+                        Err(err) => return Err(err.clone()),
+                    }
+                    let mut res = String::with_capacity(8 * 1024);
+                    if let Some(mailbox_hash) = mailbox_hash {
+                        conn.select_group(mailbox_hash, false, &mut res).await?;
+                    }
+                    conn.send_command(b"POST").await?;
+                    conn.read_response(&mut res, false, &["340 "]).await?;
+                    conn.send_multiline_data_block(&bytes).await?;
+                    conn.read_response(&mut res, false, &["240 "]).await?;
+                    Ok(())
+                }
+                Err(err) => Err(err),
+            }
+        }))
+    }
 }
 
 impl NntpType {
@@ -388,10 +544,10 @@ impl NntpType {
         */
         let server_port = get_conf_val!(s["server_port"], 119)?;
         let use_tls = get_conf_val!(s["use_tls"], server_port == 563)?;
-        let use_starttls = use_tls && get_conf_val!(s["use_starttls"], !(server_port == 563))?;
+        let use_starttls = use_tls && get_conf_val!(s["use_starttls"], false)?;
         let danger_accept_invalid_certs: bool =
             get_conf_val!(s["danger_accept_invalid_certs"], false)?;
-        let require_auth = get_conf_val!(s["require_auth"], true)?;
+        let require_auth = get_conf_val!(s["require_auth"], false)?;
         let server_conf = NntpServerConf {
             server_hostname: server_hostname.to_string(),
             server_username: if require_auth {
@@ -411,7 +567,7 @@ impl NntpType {
             danger_accept_invalid_certs,
             extension_use: NntpExtensionUse {
                 #[cfg(feature = "deflate_compression")]
-                deflate: get_conf_val!(s["use_deflate"], true)?,
+                deflate: get_conf_val!(s["use_deflate"], false)?,
             },
         };
         let account_hash = {
@@ -430,6 +586,7 @@ impl NntpType {
                     nntp_path: k.to_string(),
                     high_watermark: Arc::new(Mutex::new(0)),
                     low_watermark: Arc::new(Mutex::new(0)),
+                    latest_article: Arc::new(Mutex::new(None)),
                     exists: Default::default(),
                     unseen: Default::default(),
                 },
@@ -498,6 +655,37 @@ impl NntpType {
     }
 
     pub fn validate_config(s: &AccountSettings) -> Result<()> {
+        let mut keys: HashSet<&'static str> = Default::default();
+        macro_rules! get_conf_val {
+    ($s:ident[$var:literal]) => {{
+        keys.insert($var);
+        $s.extra.get($var).ok_or_else(|| {
+            MeliError::new(format!(
+                "Configuration error ({}): NNTP connection requires the field `{}` set",
+                $s.name.as_str(),
+                $var
+            ))
+        })
+    }};
+    ($s:ident[$var:literal], $default:expr) => {{
+        keys.insert($var);
+        $s.extra
+            .get($var)
+            .map(|v| {
+                <_>::from_str(v).map_err(|e| {
+                    MeliError::new(format!(
+                        "Configuration error ({}) NNTP: Invalid value for field `{}`: {}\n{}",
+                        $s.name.as_str(),
+                        $var,
+                        v,
+                        e
+                    ))
+                })
+            })
+            .unwrap_or_else(|| Ok($default))
+    }};
+}
+        get_conf_val!(s["require_auth"], false)?;
         get_conf_val!(s["server_hostname"])?;
         get_conf_val!(s["server_username"], String::new())?;
         if !s.extra.contains_key("server_password_command") {
@@ -518,7 +706,7 @@ impl NntpType {
             )));
         }
         #[cfg(feature = "deflate_compression")]
-        get_conf_val!(s["use_deflate"], true)?;
+        get_conf_val!(s["use_deflate"], false)?;
         #[cfg(not(feature = "deflate_compression"))]
         if s.extra.contains_key("use_deflate") {
             return Err(MeliError::new(format!(
@@ -527,6 +715,18 @@ impl NntpType {
             )));
         }
         get_conf_val!(s["danger_accept_invalid_certs"], false)?;
+        let extra_keys = s
+            .extra
+            .keys()
+            .map(String::as_str)
+            .collect::<HashSet<&str>>();
+        let diff = extra_keys.difference(&keys).collect::<Vec<&&str>>();
+        if !diff.is_empty() {
+            return Err(MeliError::new(format!(
+                "Configuration error ({}) NNTP: the following flags are set but are not recognized: {:?}.",
+                s.name.as_str(), diff
+            )));
+        }
         Ok(())
     }
 
@@ -592,10 +792,11 @@ impl FetchState {
         if high <= low {
             return Ok(None);
         }
-        const CHUNK_SIZE: usize = 100;
+        const CHUNK_SIZE: usize = 50000;
         let new_low = std::cmp::max(low, high.saturating_sub(CHUNK_SIZE));
         high_low_total.as_mut().unwrap().0 = new_low;
 
+        // FIXME: server might not implement OVER capability
         conn.send_command(format!("OVER {}-{}", new_low, high).as_bytes())
             .await?;
         conn.read_response(&mut res, true, command_to_replycodes("OVER"))
@@ -609,19 +810,28 @@ impl FetchState {
         let mut ret = Vec::with_capacity(high - new_low);
         //hash_index: Arc<Mutex<HashMap<EnvelopeHash, (UID, MailboxHash)>>>,
         //uid_index: Arc<Mutex<HashMap<(MailboxHash, UID), EnvelopeHash>>>,
+        let mut latest_article: Option<crate::UnixTimestamp> = None;
         {
+            let mut message_id_lck = uid_store.message_id_index.lock().unwrap();
             let mut hash_index_lck = uid_store.hash_index.lock().unwrap();
             let mut uid_index_lck = uid_store.uid_index.lock().unwrap();
             for l in res.split_rn().skip(1) {
                 let (_, (num, env)) = protocol_parser::over_article(&l)?;
+                message_id_lck.insert(env.message_id_display().to_string(), env.hash());
                 hash_index_lck.insert(env.hash(), (num, mailbox_hash));
                 uid_index_lck.insert((mailbox_hash, num), env.hash());
+                if let Some(ref mut v) = latest_article {
+                    *v = std::cmp::max(*v, env.timestamp);
+                } else {
+                    latest_article = Some(env.timestamp);
+                }
                 ret.push(env);
             }
         }
         {
             let hash_set: BTreeSet<EnvelopeHash> = ret.iter().map(|env| env.hash()).collect();
             let f = &uid_store.mailboxes.lock().await[&mailbox_hash];
+            *f.latest_article.lock().unwrap() = latest_article;
             f.exists
                 .lock()
                 .unwrap()
