@@ -48,6 +48,7 @@ pub enum Charset {
     Windows1253,
     GBK,
     GB2312,
+    GB18030,
     BIG5,
     ISO2022JP,
     EUCJP,
@@ -143,6 +144,9 @@ impl<'a> From<&'a [u8]> for Charset {
                 Charset::Windows1253
             }
             b if b.eq_ignore_ascii_case(b"gbk") => Charset::GBK,
+            b if b.eq_ignore_ascii_case(b"gb18030") || b.eq_ignore_ascii_case(b"gb-18030") => {
+                Charset::GB18030
+            }
             b if b.eq_ignore_ascii_case(b"gb2312") || b.eq_ignore_ascii_case(b"gb-2312") => {
                 Charset::GB2312
             }
@@ -184,6 +188,7 @@ impl Display for Charset {
             Charset::Windows1253 => write!(f, "windows-1253"),
             Charset::GBK => write!(f, "gbk"),
             Charset::GB2312 => write!(f, "gb2312"),
+            Charset::GB18030 => write!(f, "gb18030"),
             Charset::BIG5 => write!(f, "big5"),
             Charset::ISO2022JP => write!(f, "iso-2022-jp"),
             Charset::EUCJP => write!(f, "euc-jp"),
